@@ -24,6 +24,10 @@
 
 defined('MOODLE_INTERNAL') || die();
 
+global $CFG;
+require_once($CFG->dirroot . '/mod/forum/lib.php');
+require_once($CFG->dirroot . '/rating/lib.php');
+
 class mod_forum_lib_testcase extends advanced_testcase {
 
     public function test_forum_trigger_content_uploaded_event() {
@@ -36,7 +40,7 @@ class mod_forum_lib_testcase extends advanced_testcase {
 
         $this->setUser($user->id);
         $fakepost = (object) array('id' => 123, 'message' => 'Yay!', 'discussion' => 100);
-        $cm = get_coursemodule_from_instance('forum', $forum->cmid);
+        $cm = get_coursemodule_from_instance('forum', $forum->id);
 
         $fs = get_file_storage();
         $dummy = (object) array(
@@ -73,6 +77,7 @@ class mod_forum_lib_testcase extends advanced_testcase {
         $expected->content = $fakepost->message;
         $expected->pathnamehashes = array($fi->get_pathnamehash());
         $this->assertEventLegacyData($expected, $event);
+        $this->assertEventContextNotUsed($event);
     }
 
     public function test_forum_get_courses_user_posted_in() {
@@ -169,39 +174,6 @@ class mod_forum_lib_testcase extends advanced_testcase {
         }
     }
 
-    /**
-     * Test user_enrolment_deleted observer.
-     */
-    public function test_user_enrolment_deleted_observer() {
-        global $DB;
-
-        $this->resetAfterTest();
-
-        $metaplugin = enrol_get_plugin('meta');
-        $user1 = $this->getDataGenerator()->create_user();
-        $course1 = $this->getDataGenerator()->create_course();
-        $course2 = $this->getDataGenerator()->create_course();
-        $student = $DB->get_record('role', array('shortname' => 'student'));
-
-        $e1 = $metaplugin->add_instance($course2, array('customint1' => $course1->id));
-        $enrol1 = $DB->get_record('enrol', array('id' => $e1));
-
-        // Enrol user.
-        $metaplugin->enrol_user($enrol1, $user1->id, $student->id);
-        $this->assertEquals(1, $DB->count_records('user_enrolments'));
-
-        // Unenrol user and capture event.
-        $sink = $this->redirectEvents();
-        $metaplugin->unenrol_user($enrol1, $user1->id);
-        $events = $sink->get_events();
-        $sink->close();
-        $event = array_pop($events);
-
-        $this->assertEquals(0, $DB->count_records('user_enrolments'));
-        $this->assertInstanceOf('\core\event\user_enrolment_deleted', $event);
-        $this->assertEquals('user_unenrolled', $event->get_legacy_eventname());
-    }
-
     /**
      * Test the logic in the forum_tp_can_track_forums() function.
      */
@@ -607,4 +579,417 @@ class mod_forum_lib_testcase extends advanced_testcase {
         $this->assertEquals(true, isset($result[$forumoptional->id]));
         $this->assertEquals(true, isset($result[$forumforce->id]));
     }
+
+    /**
+     * Test subscription using automatic subscription on create.
+     */
+    public function test_forum_auto_subscribe_on_create() {
+        global $CFG;
+
+        $this->resetAfterTest();
+
+        $usercount = 5;
+        $course = $this->getDataGenerator()->create_course();
+        $users = array();
+
+        for ($i = 0; $i < $usercount; $i++) {
+            $user = $this->getDataGenerator()->create_user();
+            $users[] = $user;
+            $this->getDataGenerator()->enrol_user($user->id, $course->id);
+        }
+
+        $options = array('course' => $course->id, 'forcesubscribe' => FORUM_INITIALSUBSCRIBE); // Automatic Subscription.
+        $forum = $this->getDataGenerator()->create_module('forum', $options);
+
+        $result = forum_subscribed_users($course, $forum);
+        $this->assertEquals($usercount, count($result));
+        foreach ($users as $user) {
+            $this->assertTrue(forum_is_subscribed($user->id, $forum));
+        }
+    }
+
+    /**
+     * Test subscription using forced subscription on create.
+     */
+    public function test_forum_forced_subscribe_on_create() {
+        global $CFG;
+
+        $this->resetAfterTest();
+
+        $usercount = 5;
+        $course = $this->getDataGenerator()->create_course();
+        $users = array();
+
+        for ($i = 0; $i < $usercount; $i++) {
+            $user = $this->getDataGenerator()->create_user();
+            $users[] = $user;
+            $this->getDataGenerator()->enrol_user($user->id, $course->id);
+        }
+
+        $options = array('course' => $course->id, 'forcesubscribe' => FORUM_FORCESUBSCRIBE); // Forced subscription.
+        $forum = $this->getDataGenerator()->create_module('forum', $options);
+
+        $result = forum_subscribed_users($course, $forum);
+        $this->assertEquals($usercount, count($result));
+        foreach ($users as $user) {
+            $this->assertTrue(forum_is_subscribed($user->id, $forum));
+        }
+    }
+
+    /**
+     * Test subscription using optional subscription on create.
+     */
+    public function test_forum_optional_subscribe_on_create() {
+        global $CFG;
+
+        $this->resetAfterTest();
+
+        $usercount = 5;
+        $course = $this->getDataGenerator()->create_course();
+        $users = array();
+
+        for ($i = 0; $i < $usercount; $i++) {
+            $user = $this->getDataGenerator()->create_user();
+            $users[] = $user;
+            $this->getDataGenerator()->enrol_user($user->id, $course->id);
+        }
+
+        $options = array('course' => $course->id, 'forcesubscribe' => FORUM_CHOOSESUBSCRIBE); // Subscription optional.
+        $forum = $this->getDataGenerator()->create_module('forum', $options);
+
+        $result = forum_subscribed_users($course, $forum);
+        // No subscriptions by default.
+        $this->assertEquals(0, count($result));
+        foreach ($users as $user) {
+            $this->assertFalse(forum_is_subscribed($user->id, $forum));
+        }
+    }
+
+    /**
+     * Test subscription using disallow subscription on create.
+     */
+    public function test_forum_disallow_subscribe_on_create() {
+        global $CFG;
+
+        $this->resetAfterTest();
+
+        $usercount = 5;
+        $course = $this->getDataGenerator()->create_course();
+        $users = array();
+
+        for ($i = 0; $i < $usercount; $i++) {
+            $user = $this->getDataGenerator()->create_user();
+            $users[] = $user;
+            $this->getDataGenerator()->enrol_user($user->id, $course->id);
+        }
+
+        $options = array('course' => $course->id, 'forcesubscribe' => FORUM_DISALLOWSUBSCRIBE); // Subscription prevented.
+        $forum = $this->getDataGenerator()->create_module('forum', $options);
+
+        $result = forum_subscribed_users($course, $forum);
+        // No subscriptions by default.
+        $this->assertEquals(0, count($result));
+        foreach ($users as $user) {
+            $this->assertFalse(forum_is_subscribed($user->id, $forum));
+        }
+    }
+
+    public function test_count_discussion_replies_basic() {
+        list($forum, $discussionids) = $this->create_multiple_discussions_with_replies(10, 5);
+
+        // Count the discussion replies in the forum.
+        $result = forum_count_discussion_replies($forum->id);
+        $this->assertCount(10, $result);
+    }
+
+    public function test_count_discussion_replies_limited() {
+        list($forum, $discussionids) = $this->create_multiple_discussions_with_replies(10, 5);
+        // Adding limits shouldn't make a difference.
+        $result = forum_count_discussion_replies($forum->id, "", 20);
+        $this->assertCount(10, $result);
+    }
+
+    public function test_count_discussion_replies_paginated() {
+        list($forum, $discussionids) = $this->create_multiple_discussions_with_replies(10, 5);
+        // Adding paging shouldn't make any difference.
+        $result = forum_count_discussion_replies($forum->id, "", -1, 0, 100);
+        $this->assertCount(10, $result);
+    }
+
+    public function test_count_discussion_replies_paginated_sorted() {
+        list($forum, $discussionids) = $this->create_multiple_discussions_with_replies(10, 5);
+        // Specifying the forumsort should also give a good result. This follows a different path.
+        $result = forum_count_discussion_replies($forum->id, "d.id asc", -1, 0, 100);
+        $this->assertCount(10, $result);
+        foreach ($result as $row) {
+            // Grab the first discussionid.
+            $discussionid = array_shift($discussionids);
+            $this->assertEquals($discussionid, $row->discussion);
+        }
+    }
+
+    public function test_count_discussion_replies_limited_sorted() {
+        list($forum, $discussionids) = $this->create_multiple_discussions_with_replies(10, 5);
+        // Adding limits, and a forumsort shouldn't make a difference.
+        $result = forum_count_discussion_replies($forum->id, "d.id asc", 20);
+        $this->assertCount(10, $result);
+        foreach ($result as $row) {
+            // Grab the first discussionid.
+            $discussionid = array_shift($discussionids);
+            $this->assertEquals($discussionid, $row->discussion);
+        }
+    }
+
+    public function test_count_discussion_replies_paginated_sorted_small() {
+        list($forum, $discussionids) = $this->create_multiple_discussions_with_replies(10, 5);
+        // Grabbing a smaller subset and they should be ordered as expected.
+        $result = forum_count_discussion_replies($forum->id, "d.id asc", -1, 0, 5);
+        $this->assertCount(5, $result);
+        foreach ($result as $row) {
+            // Grab the first discussionid.
+            $discussionid = array_shift($discussionids);
+            $this->assertEquals($discussionid, $row->discussion);
+        }
+    }
+
+    public function test_count_discussion_replies_paginated_sorted_small_reverse() {
+        list($forum, $discussionids) = $this->create_multiple_discussions_with_replies(10, 5);
+        // Grabbing a smaller subset and they should be ordered as expected.
+        $result = forum_count_discussion_replies($forum->id, "d.id desc", -1, 0, 5);
+        $this->assertCount(5, $result);
+        foreach ($result as $row) {
+            // Grab the last discussionid.
+            $discussionid = array_pop($discussionids);
+            $this->assertEquals($discussionid, $row->discussion);
+        }
+    }
+
+    public function test_count_discussion_replies_limited_sorted_small_reverse() {
+        list($forum, $discussionids) = $this->create_multiple_discussions_with_replies(10, 5);
+        // Adding limits, and a forumsort shouldn't make a difference.
+        $result = forum_count_discussion_replies($forum->id, "d.id desc", 5);
+        $this->assertCount(5, $result);
+        foreach ($result as $row) {
+            // Grab the last discussionid.
+            $discussionid = array_pop($discussionids);
+            $this->assertEquals($discussionid, $row->discussion);
+        }
+    }
+
+    /**
+     * Create a new course, forum, and user with a number of discussions and replies.
+     *
+     * @param int $discussioncount The number of discussions to create
+     * @param int $replycount The number of replies to create in each discussion
+     * @return array Containing the created forum object, and the ids of the created discussions.
+     */
+    protected function create_multiple_discussions_with_replies($discussioncount, $replycount) {
+        $this->resetAfterTest();
+
+        // Setup the content.
+        $user = $this->getDataGenerator()->create_user();
+        $course = $this->getDataGenerator()->create_course();
+        $record = new stdClass();
+        $record->course = $course->id;
+        $forum = $this->getDataGenerator()->create_module('forum', $record);
+
+        // Create 10 discussions with replies.
+        $discussionids = array();
+        for ($i = 0; $i < $discussioncount; $i++) {
+            $discussion = $this->create_single_discussion_with_replies($forum, $user, $replycount);
+            $discussionids[] = $discussion->id;
+        }
+        return array($forum, $discussionids);
+    }
+
+    /**
+     * Create a discussion with a number of replies.
+     *
+     * @param object $forum The forum which has been created
+     * @param object $user The user making the discussion and replies
+     * @param int $replycount The number of replies
+     * @return object $discussion
+     */
+    protected function create_single_discussion_with_replies($forum, $user, $replycount) {
+        global $DB;
+
+        $generator = self::getDataGenerator()->get_plugin_generator('mod_forum');
+
+        $record = new stdClass();
+        $record->course = $forum->course;
+        $record->forum = $forum->id;
+        $record->userid = $user->id;
+        $discussion = $generator->create_discussion($record);
+
+        // Retrieve the first post.
+        $replyto = $DB->get_record('forum_posts', array('discussion' => $discussion->id));
+
+        // Create the replies.
+        $post = new stdClass();
+        $post->userid = $user->id;
+        $post->discussion = $discussion->id;
+        $post->parent = $replyto->id;
+
+        for ($i = 0; $i < $replycount; $i++) {
+            $generator->create_post($post);
+        }
+
+        return $discussion;
+    }
+
+    /**
+     * Tests for mod_forum_rating_can_see_item_ratings().
+     *
+     * @throws coding_exception
+     * @throws rating_exception
+     */
+    public function test_mod_forum_rating_can_see_item_ratings() {
+        global $DB;
+
+        $this->resetAfterTest();
+
+        // Setup test data.
+        $course = new stdClass();
+        $course->groupmode = SEPARATEGROUPS;
+        $course->groupmodeforce = true;
+        $course = $this->getDataGenerator()->create_course($course);
+        $forum = $this->getDataGenerator()->create_module('forum', array('course' => $course->id));
+        $generator = self::getDataGenerator()->get_plugin_generator('mod_forum');
+        $cm = get_coursemodule_from_instance('forum', $forum->id);
+        $context = context_module::instance($cm->id);
+
+        // Create users.
+        $user1 = $this->getDataGenerator()->create_user();
+        $user2 = $this->getDataGenerator()->create_user();
+        $user3 = $this->getDataGenerator()->create_user();
+        $user4 = $this->getDataGenerator()->create_user();
+
+        // Groups and stuff.
+        $role = $DB->get_record('role', array('shortname' => 'teacher'), '*', MUST_EXIST);
+        $this->getDataGenerator()->enrol_user($user1->id, $course->id, $role->id);
+        $this->getDataGenerator()->enrol_user($user2->id, $course->id, $role->id);
+        $this->getDataGenerator()->enrol_user($user3->id, $course->id, $role->id);
+        $this->getDataGenerator()->enrol_user($user4->id, $course->id, $role->id);
+
+        $group1 = $this->getDataGenerator()->create_group(array('courseid' => $course->id));
+        $group2 = $this->getDataGenerator()->create_group(array('courseid' => $course->id));
+        groups_add_member($group1, $user1);
+        groups_add_member($group1, $user2);
+        groups_add_member($group2, $user3);
+        groups_add_member($group2, $user4);
+
+        $record = new stdClass();
+        $record->course = $forum->course;
+        $record->forum = $forum->id;
+        $record->userid = $user1->id;
+        $record->groupid = $group1->id;
+        $discussion = $generator->create_discussion($record);
+
+        // Retrieve the first post.
+        $post = $DB->get_record('forum_posts', array('discussion' => $discussion->id));
+
+        $ratingoptions = new stdClass;
+        $ratingoptions->context = $context;
+        $ratingoptions->ratingarea = 'post';
+        $ratingoptions->component = 'mod_forum';
+        $ratingoptions->itemid  = $post->id;
+        $ratingoptions->scaleid = 2;
+        $ratingoptions->userid  = $user2->id;
+        $rating = new rating($ratingoptions);
+        $rating->update_rating(2);
+
+        // Now try to access it as various users.
+        unassign_capability('moodle/site:accessallgroups', $role->id);
+        $params = array('contextid' => 2,
+                        'component' => 'mod_forum',
+                        'ratingarea' => 'post',
+                        'itemid' => $post->id,
+                        'scaleid' => 2);
+        $this->setUser($user1);
+        $this->assertTrue(mod_forum_rating_can_see_item_ratings($params));
+        $this->setUser($user2);
+        $this->assertTrue(mod_forum_rating_can_see_item_ratings($params));
+        $this->setUser($user3);
+        $this->assertFalse(mod_forum_rating_can_see_item_ratings($params));
+        $this->setUser($user4);
+        $this->assertFalse(mod_forum_rating_can_see_item_ratings($params));
+
+        // Now try with accessallgroups cap and make sure everything is visible.
+        assign_capability('moodle/site:accessallgroups', CAP_ALLOW, $role->id, $context->id);
+        $this->setUser($user1);
+        $this->assertTrue(mod_forum_rating_can_see_item_ratings($params));
+        $this->setUser($user2);
+        $this->assertTrue(mod_forum_rating_can_see_item_ratings($params));
+        $this->setUser($user3);
+        $this->assertTrue(mod_forum_rating_can_see_item_ratings($params));
+        $this->setUser($user4);
+        $this->assertTrue(mod_forum_rating_can_see_item_ratings($params));
+
+        // Change group mode and verify visibility.
+        $course->groupmode = VISIBLEGROUPS;
+        $DB->update_record('course', $course);
+        unassign_capability('moodle/site:accessallgroups', $role->id);
+        $this->setUser($user1);
+        $this->assertTrue(mod_forum_rating_can_see_item_ratings($params));
+        $this->setUser($user2);
+        $this->assertTrue(mod_forum_rating_can_see_item_ratings($params));
+        $this->setUser($user3);
+        $this->assertTrue(mod_forum_rating_can_see_item_ratings($params));
+        $this->setUser($user4);
+        $this->assertTrue(mod_forum_rating_can_see_item_ratings($params));
+
+    }
+
+    /**
+     * Test that {@link forum_update_post()} keeps correct forum_discussions usermodified.
+     */
+    public function test_forum_update_post_keeps_discussions_usermodified() {
+        global $DB;
+
+        $this->resetAfterTest();
+
+        // Let there be light.
+        $teacher = self::getDataGenerator()->create_user();
+        $student = self::getDataGenerator()->create_user();
+        $course = self::getDataGenerator()->create_course();
+
+        $forum = self::getDataGenerator()->create_module('forum', (object)[
+            'course' => $course->id,
+        ]);
+
+        $generator = self::getDataGenerator()->get_plugin_generator('mod_forum');
+
+        // Let the teacher start a discussion.
+        $discussion = $generator->create_discussion((object)[
+            'course' => $course->id,
+            'userid' => $teacher->id,
+            'forum' => $forum->id,
+        ]);
+
+        // On this freshly created discussion, the teacher is the author of the last post.
+        $this->assertEquals($teacher->id, $DB->get_field('forum_discussions', 'usermodified', ['id' => $discussion->id]));
+
+        // Let the student reply to the teacher's post.
+        $reply = $generator->create_post((object)[
+            'course' => $course->id,
+            'userid' => $student->id,
+            'forum' => $forum->id,
+            'discussion' => $discussion->id,
+            'parent' => $discussion->firstpost,
+        ]);
+        // The student should now be the last post's author.
+        $this->assertEquals($student->id, $DB->get_field('forum_discussions', 'usermodified', ['id' => $discussion->id]));
+
+        // Let the teacher edit the student's reply.
+        $this->setUser($teacher->id);
+        $newpost = (object)[
+            'id' => $reply->id,
+            'itemid' => 0,
+            'subject' => 'Amended subject',
+        ];
+        forum_update_post($newpost, null, $message);
+
+        // The student should be still the last post's author.
+        $this->assertEquals($student->id, $DB->get_field('forum_discussions', 'usermodified', ['id' => $discussion->id]));
+    }
 }
