@@ -18,8 +18,7 @@
 /**
  * Private url module utility functions
  *
- * @package    mod
- * @subpackage url
+ * @package    mod_url
  * @copyright  2009 Petr Skoda  {@link http://skodak.org}
  * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
  */
@@ -40,9 +39,9 @@ require_once("$CFG->dirroot/mod/url/lib.php");
 function url_appears_valid_url($url) {
     if (preg_match('/^(\/|https?:|ftp:)/i', $url)) {
         // note: this is not exact validation, we look for severely malformed URLs only
-        return preg_match('/^[a-z]+:\/\/([^:@\s]+:[^@\s]+@)?[a-z0-9_\.\-]+(:[0-9]+)?(\/[^#]*)?(#.*)?$/i', $url);
+        return (bool)preg_match('/^[a-z]+:\/\/([^:@\s]+:[^@\s]+@)?[a-z0-9_\.\-]+(:[0-9]+)?(\/[^#]*)?(#.*)?$/i', $url);
     } else {
-        return preg_match('/^[a-z]+:\/\/...*$/i', $url);
+        return (bool)preg_match('/^[a-z]+:\/\/...*$/i', $url);
     }
 }
 
@@ -121,7 +120,7 @@ function url_get_full_url($url, $cm, $course, $config=null) {
             if (stripos($fullurl, 'teamspeak://') === 0) {
                 $fullurl = $fullurl.'?'.implode('?', $parameters);
             } else {
-                $join = (strpos($fullurl->externalurl, '?') === false) ? '?' : '&';
+                $join = (strpos($fullurl, '?') === false) ? '?' : '&';
                 $fullurl = $fullurl.$join.implode('&', $parameters);
             }
         }
@@ -164,17 +163,12 @@ function url_print_header($url, $cm, $course) {
  * @param object $url
  * @param object $cm
  * @param object $course
- * @param bool $ignoresettings print even if not specified in modedit
+ * @param bool $notused This variable is no longer used.
  * @return void
  */
-function url_print_heading($url, $cm, $course, $ignoresettings=false) {
+function url_print_heading($url, $cm, $course, $notused = false) {
     global $OUTPUT;
-
-    $options = empty($url->displayoptions) ? array() : unserialize($url->displayoptions);
-
-    if ($ignoresettings or !empty($options['printheading'])) {
-        echo $OUTPUT->heading(format_string($url->name), 2, 'main', 'urlheading');
-    }
+    echo $OUTPUT->heading(format_string($url->name), 2);
 }
 
 /**
@@ -220,14 +214,15 @@ function url_display_frame($url, $cm, $course) {
 
     } else {
         $config = get_config('url');
-        $context = get_context_instance(CONTEXT_MODULE, $cm->id);
+        $context = context_module::instance($cm->id);
         $exteurl = url_get_full_url($url, $cm, $course, $config);
         $navurl = "$CFG->wwwroot/mod/url/view.php?id=$cm->id&amp;frameset=top";
-        $coursecontext = get_context_instance(CONTEXT_COURSE, $course->id);
+        $coursecontext = context_course::instance($course->id);
         $courseshortname = format_string($course->shortname, true, array('context' => $coursecontext));
         $title = strip_tags($courseshortname.': '.format_string($url->name));
         $framesize = $config->framesize;
         $modulename = s(get_string('modulename','url'));
+        $contentframetitle = format_string($url->name);
         $dir = get_string('thisdirection', 'langconfig');
 
         $extframe = <<<EOF
@@ -239,7 +234,7 @@ function url_display_frame($url, $cm, $course) {
   </head>
   <frameset rows="$framesize,*">
     <frame src="$navurl" title="$modulename"/>
-    <frame src="$exteurl" title="$modulename"/>
+    <frame src="$exteurl" title="$contentframetitle"/>
   </frameset>
 </html>
 EOF;
@@ -306,39 +301,22 @@ function url_display_embed($url, $cm, $course) {
 
     $link = html_writer::tag('a', $fullurl, array('href'=>str_replace('&amp;', '&', $fullurl)));
     $clicktoopen = get_string('clicktoopen', 'url', $link);
+    $moodleurl = new moodle_url($fullurl);
 
     $extension = resourcelib_get_extension($url->externalurl);
 
+    $mediarenderer = $PAGE->get_renderer('core', 'media');
+    $embedoptions = array(
+        core_media::OPTION_TRUSTED => true,
+        core_media::OPTION_BLOCK => true
+    );
+
     if (in_array($mimetype, array('image/gif','image/jpeg','image/png'))) {  // It's an image
         $code = resourcelib_embed_image($fullurl, $title);
 
-    } else if ($mimetype == 'audio/mp3') {
-        // MP3 audio file
-        $code = resourcelib_embed_mp3($fullurl, $title, $clicktoopen);
-
-    } else if ($mimetype == 'video/x-flv' or $extension === 'f4v') {
-        // Flash video file
-        $code = resourcelib_embed_flashvideo($fullurl, $title, $clicktoopen);
-
-    } else if ($mimetype == 'application/x-shockwave-flash') {
-        // Flash file
-        $code = resourcelib_embed_flash($fullurl, $title, $clicktoopen);
-
-    } else if (substr($mimetype, 0, 10) == 'video/x-ms') {
-        // Windows Media Player file
-        $code = resourcelib_embed_mediaplayer($fullurl, $title, $clicktoopen);
-
-    } else if ($mimetype == 'video/quicktime') {
-        // Quicktime file
-        $code = resourcelib_embed_quicktime($fullurl, $title, $clicktoopen);
-
-    } else if ($mimetype == 'video/mpeg') {
-        // Mpeg file
-        $code = resourcelib_embed_mpeg($fullurl, $title, $clicktoopen);
-
-    } else if ($mimetype == 'audio/x-pn-realaudio-plugin') {
-        // RealMedia file
-        $code = resourcelib_embed_real($fullurl, $title, $clicktoopen);
+    } else if ($mediarenderer->can_embed_url($moodleurl, $embedoptions)) {
+        // Media (audio/video) file.
+        $code = $mediarenderer->embed_url($moodleurl, $title, 0, 0, $embedoptions);
 
     } else {
         // anything else - just try object tag enlarged as much as possible
@@ -454,10 +432,10 @@ function url_get_variable_options($config) {
     );
 
     if ($config->rolesinparams) {
-        $roles = get_all_roles();
+        $roles = role_fix_names(get_all_roles());
         $roleoptions = array();
         foreach ($roles as $role) {
-            $roleoptions['course'.$role->shortname] = get_string('yourwordforx', '', $role->name);
+            $roleoptions['course'.$role->shortname] = get_string('yourwordforx', '', $role->localname);
         }
         $options[get_string('roles')] = $roleoptions;
     }
@@ -478,7 +456,7 @@ function url_get_variable_values($url, $cm, $course, $config) {
 
     $site = get_site();
 
-    $coursecontext = get_context_instance(CONTEXT_COURSE, $course->id);
+    $coursecontext = context_course::instance($course->id);
 
     $values = array (
         'courseid'        => $course->id,
@@ -525,9 +503,8 @@ function url_get_variable_values($url, $cm, $course, $config) {
 
     //hmm, this is pretty fragile and slow, why do we need it here??
     if ($config->rolesinparams) {
-        $roles = get_all_roles();
-        $coursecontext = get_context_instance(CONTEXT_COURSE, $course->id);
-        $roles = role_fix_names($roles, $coursecontext, ROLENAME_ALIAS);
+        $coursecontext = context_course::instance($course->id);
+        $roles = role_fix_names(get_all_roles($coursecontext), $coursecontext, ROLENAME_ALIAS);
         foreach ($roles as $role) {
             $values['course'.$role->shortname] = $role->localname;
         }
@@ -557,22 +534,26 @@ function url_get_encrypted_parameter($url, $config) {
 /**
  * Optimised mimetype detection from general URL
  * @param $fullurl
- * @return string mimetype
+ * @param int $size of the icon.
+ * @return string|null mimetype or null when the filetype is not relevant.
  */
-function url_guess_icon($fullurl) {
+function url_guess_icon($fullurl, $size = null) {
     global $CFG;
     require_once("$CFG->libdir/filelib.php");
 
     if (substr_count($fullurl, '/') < 3 or substr($fullurl, -1) === '/') {
-        // most probably default directory - index.php, index.html, etc.
-        return 'f/web';
+        // Most probably default directory - index.php, index.html, etc. Return null because
+        // we want to use the default module icon instead of the HTML file icon.
+        return null;
     }
 
-    $icon = mimeinfo('icon', $fullurl);
-    $icon = 'f/'.str_replace(array('.gif', '.png'), '', $icon);
+    $icon = file_extension_icon($fullurl, $size);
+    $htmlicon = file_extension_icon('.htm', $size);
+    $unknownicon = file_extension_icon('', $size);
 
-    if ($icon === 'f/html' or $icon === 'f/unknown') {
-        $icon = 'f/web';
+    // We do not want to return those icon types, the module icon is more appropriate.
+    if ($icon === $unknownicon || $icon === $htmlicon) {
+        return null;
     }
 
     return $icon;
